// Mode1 PIDs
//
enum eM1PID
{							// PID, #bytes, freq
	eM1Pid_SuppMaskPage1,	// 00 4 i  PIDs supported
	eM1Pid_Diag,			// 01 4 i  Diag complete, MIL, DTC count
	eM1Pid_Freeze,			// 02 8 i  Freeze DTC
	eM1Pid_FuelSystem,		// 03 2 i
	eM1Pid_EngineLoad,		// 04 1 1
	eM1Pid_CoolantTemp,		// 05 1
	eM1Pid_STFT1,			// 06 1 1
	eM1Pid_LTFT1,			// 07 1 60
	eM1Pid_STFT2,			// 08 1 1
	eM1Pid_LTFT2,			// 09 1 60
	eM1Pid_FuelPressure,	// 0A 1 1
	eM1Pid_IMP,				// 0B 1
	eM1Pid_RPM,				// 0C 2 1
	eM1Pid_KPH,				// 0D 1 1
	eM1Pid_Timing,			// 0E 1 1
	eM1Pid_AIT,				// 0F 1 40
	eM1Pid_MAFRate,			// 10 2 1
	eM1Pid_TPS,				// 11 1 1
	eM1Pid_SecAirStatus,	// 12 1 ?
	eM1Pid_O2Count,			// 13 2 i
	eM1Pid_O2B1S1,			// 14 2
	eM1Pid_O2B1S2,			// 15 2
	eM1Pid_O2B1S3,			// 16 2
	eM1Pid_O2B1S4,			// 17 2
	eM1Pid_O2B2S1,			// 18 2
	eM1Pid_O2B2S2,			// 19 2
	eM1Pid_O2B2S3,			// 1A 2
	eM1Pid_O2B2S4,			// 1B 2
	eM1Pid_OBDStdMask,		// 1C 1 i
	eM1Pid_O2BitMask,		// 1D 1 i
	eM1Pid_AuxInputStatus,	// 1E 1
	eM1Pid_RunTimeSecs,		// 1F 2
	eM1Pid_SuppMaskPage2,	// 20 4
	eM1Pid_DistwMIL,		// 21 2
	eM1Pid_FRP,				// 22 2
	eM1Pid_FRP_Diesel,		// 23 2
	eM1Pid_O2S1_WR,			// 24 4
	eM1Pid_O2S2_WR,			// 25 4
	eM1Pid_O2S3_WR,			// 26 4
	eM1Pid_O2S4_WR,			// 27 4
	eM1Pid_O2S5_WR,			// 28 4
	eM1Pid_O2S6_WR,			// 29 4
	eM1Pid_O2S7_WR,			// 2A 4
	eM1Pid_O2S8_WR,			// 2B 4
	eM1Pid_CEGR,			// 2C 1
	eM1Pid_EGR_Error,		// 2D 1
	eM1Pid_CEP,				// 2E 1
	eM1Pid_FLI,				// 2F 1
	eM1Pid_WarmupsClear,	// 30 1
	eM1Pid_DistSinceClear,	// 31 2
	eM1Pid_EVP,				// 32 2
	eM1Pid_Baro,			// 33 1
};

// Mode 6 storage
typedef struct _Mode6Val
{
	BYTE Flags;
	short wValue;	// signed values
	short wLimit;
} Mode6Val;

typedef struct _Mode6Gp
{
	BYTE	 Count;		// Number of valid responses
	Mode6Val v[8];
} Mode6Gp;

// Current state of ELM327 interpreter (bState)
enum ELMState
{
	ELMState_NotConnected,
	ELMState_Closed,
	ELMState_Open,
	ELMState_Responsive,
	ELMState_Initializing,
	ELMState_Timeout,
	ELMState_Busy,
	ELMState_Idle,
};

// Flags for bFlags in share
#define XOBDFlag_Volts 1

// Flags for SendCommand
#define XSCFlag_DisableReports				0	// Disable reporting
#define XSCFlag_EnableCycleReports			1	// Report on completion of cycle (Response:wParam=0, lParam=0)
#define XSCFlag_EnableIndividualReports		2	// Report on completion of each Mode1 result (Response:wParam=MODE | PID, lParam=Data)
#define XSCFlag_RequestPIDReport			4	// Request 1 PID:dwData=MAKEDWORD(Mode, Pid)

// Memory mapped storage
typedef struct _XOBD2Share
{
	BYTE	ReferenceCount;					// Memory map reference count
	BYTE	ActiveComPort;					// COM port # being used
	BYTE	bState;							// State of OBD2 communication: See enum ELMState
	BYTE	LastCompletedPID[2];			// [Mode, PID]
	BYTE	bCounter[2];					// Queue size, cycle counter
	BYTE	bFlags;							// see XOBDFlag defines
	DWORD	dwMode1PidMask[8];				// 256 available PIDs [0]bit1=PID1,[0]bit2=PID2...
	DWORD	dwMode1PidQueue[8];				// PIDs in queue to be sent
	WORD	wMode1[256][2];					// Mode1 data store: wMode[x][0] = (A*256)+B, etc junk
	WORD	wDTCCount;						// Count of current DTCs
	WORD	wDTC[24];						// List of DTC
	DWORD	dwMode5PidMask;					// Supported mode 5 PIDs
	WORD	wMode5[8][4];					// Data store for mode 5
	DWORD	dwMode6PidMask;					// Supported mode 6 PIDs
	Mode6Gp Mode6[32];						// Data store for mode 6
	DWORD	dwMode8PidMask;					// Supported mode 8 PIDs
	DWORD	dwReserved[16];					// Reserved for mode 8
	DWORD	dwODO;							// Total trip in miles * 10 if speed PID active
	DWORD	dwTrip;							// Current trip in miles * 10 if speed PID active
	WORD	wVolts;							// Last volts * 10 response (use bFlags |= XOBDFlag_Volts)
	WORD	wMPG;							// Current instantanious MPG * 10 (set PIDs 07, 0D, 10)
	WORD	wSTMPG;							// Short term averaged MPG * 10    " "
	WORD	wLTMPG;							// Long term averaged MPG * 10     " "
	WORD	wMAFAdj;						// MAF adjust to correct MPG (100 = 1:1)
	BYTE	bPIDFreq[32*8];					// PID Skipping frequency
	char	szInit[256];					// Init command list
	DWORD	dwReserved2[16];
} XOBD2Share;

// Note: For best performance, set shared memory Mode1 PIDs before returning from WM_USER dispatch
